@@ -1,32 +1,16 @@
-# Copyright (c) 2019 Vlad Balmos <vladbalmos@yahoo.com>
+# Copyright (c) 2021 Vlad Balmos <vladbalmos@yahoo.com>
 # Author: Vlad Balmos <vladbalmos@yahoo.com>
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy of
-# this software and associated documentation files (the "Software"), to deal in
-# the Software without restriction, including without limitation the rights to
-# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-# the Software, and to permit persons to whom the Software is furnished to do so,
-# subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
-# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
-# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
-# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
-# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+# See LICENSE file
 
 from collections import OrderedDict
 
 import urwid
 import mysql.connector.errors as errors
 
-import mitzasql.db.schema as schema
-from mitzasql.db.schema_cache import schema_cache_instance
+from . import schema
+from .schema_cache import schema_cache_instance
 
-MAX_ROWS_FOR_COLUMN_LENGTH_DETECTION = 100
+MAX_ROWS_FOR_COLUMN_LENGTH_DETECTION = 50
 
 class Model:
     SIGNAL_PRE_LOAD = 'preload'
@@ -97,6 +81,10 @@ class MysqlModel(Model):
         super().__init__()
         self._fetch_data()
 
+    @property
+    def connection(self):
+        return self._con
+
     @property
     def columns(self):
         # get cached widths
@@ -248,6 +236,7 @@ class TableModel(MysqlModel):
     '''Model for a database table/view'''
     def __init__(self, connection, table_name):
         self.table_name = table_name
+        self._database = connection.database
         self.schema_error = None
 
         try:
@@ -264,9 +253,24 @@ class TableModel(MysqlModel):
         self._where = None
         super().__init__(connection)
 
+
+        self.loaded_rowcount = 0
+        self.rowcount = 0
+
+        if self.last_error:
+            return
+
         self.loaded_rowcount = len(self.data)
         self.rowcount = self._count_rows()
 
+    @property
+    def database(self):
+        return self._database
+
+    @database.setter
+    def database(self, db):
+        self._database = db
+
     def _fetch_data(self):
         result = super()._fetch_data()
         return result
@@ -393,9 +397,9 @@ class TableModel(MysqlModel):
         select_columns = []
         for column, info in self._table_schema:
             if self._table_schema.column_is_text(info['type']) and info['max_len'] is not None and info['max_len'] > 256:
-                column = 'LEFT(`{0}`, 256) as {1}'.format(column, column)
+                column = 'LEFT(`{0}`, 256) as `{1}`'.format(column, column)
             elif self._table_schema.column_is_spatial(info['type']):
-                column = 'ST_AsText(`{0}`) as {1}'.format(column, column)
+                column = 'ST_AsText(`{0}`) as `{1}`'.format(column, column)
             else:
                 column = '`{0}`'.format(column)
             select_columns.append(column)
@@ -420,32 +424,42 @@ class QueryModel(MysqlModel):
         self.has_rows = False
         super().__init__(connection)
 
-    def _query_db(self):
-        cursor = self.execute_query(self.query)
-        if cursor is not None:
-            self.has_rows = cursor.with_rows
-        return cursor
+    def execute_query(self, query, params=None):
+        try:
+            cursors = self._con.query(query, params, multi=True)
+            self.last_error = None
+            return cursors
+        except errors.Error as e:
+            self.last_error = e
+            urwid.emit_signal(self, self.SIGNAL_ERROR, self, e)
 
     def _fetch_data(self):
-        cursor = self._query_db()
-        if cursor is None:
+        cursors = self.execute_query(self.query)
+
+        if cursors:
+            try:
+                for cursor in cursors:
+                    if cursor.with_rows:
+                        self.data = cursor.fetchall()
+                        self._columns = self._schema(cursor).columns
+                        self.rowcount = cursor.rowcount
+                        self.affected_rows = 0
+                        self.has_rows = cursor.with_rows
+                    else:
+                        self.data = []
+                        self._columns = []
+                        self.rowcount = 0
+                        self.affected_rows = cursor.rowcount
+            except errors.Error as e:
+                self.last_error = e
+                urwid.emit_signal(self, self.SIGNAL_ERROR, self, e)
+
+        if self.last_error:
             self.data = []
             self._columns = []
             self.rowcount = 0
             self.affected_rows = 0
-            return cursor is not None
-
-        if cursor.with_rows:
-            self.data = cursor.fetchall()
-            self._columns = self._schema(cursor).columns
-            self.rowcount = cursor.rowcount
-            self.affected_rows = 0
-        else:
-            self.data = []
-            self._columns = []
-            self.rowcount = 0
-            self.affected_rows = cursor.rowcount
-
+            return False
         return True
 
 class TriggerModel:
